@@ -40,23 +40,29 @@
 #include "Frame.h"
 #include "FrameLoader.h"
 #include "HTMLFrameOwnerElement.h"
+#include "InjectedScript.h"
+#include "InjectedScriptHost.h"
 #include "InspectorClient.h"
 #include "InspectorController.h"
 #include "InspectorDOMAgent.h"
 #include "InspectorFrontend.h"
 #include "InspectorResource.h"
-#include "ScriptFunctionCall.h"
+#include "Pasteboard.h"
+#include "ScriptArray.h"
+#include "SerializedScriptValue.h"
 
 #if ENABLE(DOM_STORAGE)
 #include "Storage.h"
 #endif
 
-#if ENABLE(JAVASCRIPT_DEBUGGER)
+#if ENABLE(JAVASCRIPT_DEBUGGER) && USE(JSC)
 #include "JavaScriptCallFrame.h"
 #include "JavaScriptDebugServer.h"
 using namespace JSC;
 #endif
 
+#include "markup.h"
+
 #include <wtf/RefPtr.h>
 #include <wtf/StdLibExtras.h>
 
@@ -64,9 +70,8 @@ using namespace std;
 
 namespace WebCore {
 
-InspectorBackend::InspectorBackend(InspectorController* inspectorController, InspectorClient* client)
+InspectorBackend::InspectorBackend(InspectorController* inspectorController)
     : m_inspectorController(inspectorController)
-    , m_client(client)
 {
 }
 
@@ -74,111 +79,10 @@ InspectorBackend::~InspectorBackend()
 {
 }
 
-void InspectorBackend::hideDOMNodeHighlight()
+void InspectorBackend::saveFrontendSettings(const String& settings)
 {
     if (m_inspectorController)
-        m_inspectorController->hideHighlight();
-}
-
-String InspectorBackend::localizedStringsURL()
-{
-    return m_client->localizedStringsURL();
-}
-
-String InspectorBackend::hiddenPanels()
-{
-    return m_client->hiddenPanels();
-}
-
-void InspectorBackend::windowUnloading()
-{
-    if (m_inspectorController)
-        m_inspectorController->close();
-}
-
-bool InspectorBackend::isWindowVisible()
-{
-    if (m_inspectorController)
-        return m_inspectorController->windowVisible();
-    return false;
-}
-
-void InspectorBackend::addResourceSourceToFrame(long identifier, Node* frame)
-{
-    if (!m_inspectorController)
-        return;
-    RefPtr<InspectorResource> resource = m_inspectorController->resources().get(identifier);
-    if (resource) {
-        String sourceString = resource->sourceString();
-        if (!sourceString.isEmpty())
-            addSourceToFrame(resource->mimeType(), sourceString, frame);
-    }
-}
-
-bool InspectorBackend::addSourceToFrame(const String& mimeType, const String& source, Node* frameNode)
-{
-    ASSERT_ARG(frameNode, frameNode);
-
-    if (!frameNode)
-        return false;
-
-    if (!frameNode->attached()) {
-        ASSERT_NOT_REACHED();
-        return false;
-    }
-
-    ASSERT(frameNode->isElementNode());
-    if (!frameNode->isElementNode())
-        return false;
-
-    Element* element = static_cast<Element*>(frameNode);
-    ASSERT(element->isFrameOwnerElement());
-    if (!element->isFrameOwnerElement())
-        return false;
-
-    HTMLFrameOwnerElement* frameOwner = static_cast<HTMLFrameOwnerElement*>(element);
-    ASSERT(frameOwner->contentFrame());
-    if (!frameOwner->contentFrame())
-        return false;
-
-    FrameLoader* loader = frameOwner->contentFrame()->loader();
-
-    loader->setResponseMIMEType(mimeType);
-    loader->begin();
-    loader->write(source);
-    loader->end();
-
-    return true;
-}
-
-void InspectorBackend::clearMessages(bool clearUI)
-{
-    if (m_inspectorController)
-        m_inspectorController->clearConsoleMessages(clearUI);
-}
-
-void InspectorBackend::toggleNodeSearch()
-{
-    if (m_inspectorController)
-        m_inspectorController->toggleSearchForNodeInPage();
-}
-
-void InspectorBackend::attach()
-{
-    if (m_inspectorController)
-        m_inspectorController->attachWindow();
-}
-
-void InspectorBackend::detach()
-{
-    if (m_inspectorController)
-        m_inspectorController->detachWindow();
-}
-
-void InspectorBackend::setAttachedWindowHeight(unsigned height)
-{
-    if (m_inspectorController)
-        m_inspectorController->setAttachedWindowHeight(height);
+        m_inspectorController->setSetting(InspectorController::FrontendSettingsSettingName, settings);
 }
 
 void InspectorBackend::storeLastActivePanel(const String& panelName)
@@ -187,6 +91,12 @@ void InspectorBackend::storeLastActivePanel(const String& panelName)
         m_inspectorController->storeLastActivePanel(panelName);
 }
 
+void InspectorBackend::toggleNodeSearch()
+{
+    if (m_inspectorController)
+        m_inspectorController->toggleSearchForNodeInPage();
+}
+
 bool InspectorBackend::searchingForNode()
 {
     if (m_inspectorController)
@@ -194,10 +104,11 @@ bool InspectorBackend::searchingForNode()
     return false;
 }
 
-void InspectorBackend::loaded()
+bool InspectorBackend::resourceTrackingEnabled() const
 {
     if (m_inspectorController)
-        m_inspectorController->scriptObjectReady();
+        return m_inspectorController->resourceTrackingEnabled();
+    return false;
 }
 
 void InspectorBackend::enableResourceTracking(bool always)
@@ -212,116 +123,36 @@ void InspectorBackend::disableResourceTracking(bool always)
         m_inspectorController->disableResourceTracking(always);
 }
 
-bool InspectorBackend::resourceTrackingEnabled() const
+void InspectorBackend::getResourceContent(long callId, unsigned long identifier)
 {
-    if (m_inspectorController)
-        return m_inspectorController->resourceTrackingEnabled();
-    return false;
-}
-
-void InspectorBackend::moveWindowBy(float x, float y) const
-{
-    if (m_inspectorController)
-        m_inspectorController->moveWindowBy(x, y);
-}
-
-void InspectorBackend::closeWindow()
-{
-    if (m_inspectorController)
-        m_inspectorController->closeWindow();
-}
-
-const String& InspectorBackend::platform() const
-{
-#if PLATFORM(MAC)
-#ifdef BUILDING_ON_TIGER
-    DEFINE_STATIC_LOCAL(const String, platform, ("mac-tiger"));
-#else
-    DEFINE_STATIC_LOCAL(const String, platform, ("mac-leopard"));
-#endif
-#elif PLATFORM(WIN_OS)
-    DEFINE_STATIC_LOCAL(const String, platform, ("windows"));
-#elif PLATFORM(QT)
-    DEFINE_STATIC_LOCAL(const String, platform, ("qt"));
-#elif PLATFORM(GTK)
-    DEFINE_STATIC_LOCAL(const String, platform, ("gtk"));
-#elif PLATFORM(WX)
-    DEFINE_STATIC_LOCAL(const String, platform, ("wx"));
-#else
-    DEFINE_STATIC_LOCAL(const String, platform, ("unknown"));
-#endif
-
-    return platform;
-}
-
-void InspectorBackend::enableTimeline(bool always)
-{
-    if (m_inspectorController)
-        m_inspectorController->enableTimeline(always);
-}
-
-void InspectorBackend::disableTimeline(bool always)
-{
-    if (m_inspectorController)
-        m_inspectorController->disableTimeline(always);
-}
-
-bool InspectorBackend::timelineEnabled() const
-{
-    if (m_inspectorController)
-        return m_inspectorController->timelineEnabled();
-    return false;
-}
-
-void InspectorBackend::getCookies(long callId)
-{
-    if (InspectorDOMAgent* domAgent = inspectorDOMAgent())
-        domAgent->getCookies(callId);
-}
-
-void InspectorBackend::deleteCookie(const String& cookieName)
-{
-    if (!m_inspectorController)
+    InspectorFrontend* frontend = inspectorFrontend();
+    if (!frontend)
         return;
-    m_inspectorController->deleteCookie(cookieName);
+
+    RefPtr<InspectorResource> resource = m_inspectorController->resources().get(identifier);
+    if (resource)
+        frontend->didGetResourceContent(callId, resource->sourceString());
+    else
+        frontend->didGetResourceContent(callId, "");
 }
 
-#if ENABLE(JAVASCRIPT_DEBUGGER)
-const ProfilesArray& InspectorBackend::profiles() const
+void InspectorBackend::startTimelineProfiler()
 {
     if (m_inspectorController)
-        return m_inspectorController->profiles();
-    return m_emptyProfiles;
+        m_inspectorController->startTimelineProfiler();
 }
 
-void InspectorBackend::startProfiling()
+void InspectorBackend::stopTimelineProfiler()
 {
     if (m_inspectorController)
-        m_inspectorController->startUserInitiatedProfiling();
+        m_inspectorController->stopTimelineProfiler();
 }
 
-void InspectorBackend::stopProfiling()
+#if ENABLE(JAVASCRIPT_DEBUGGER) && USE(JSC)
+bool InspectorBackend::debuggerEnabled() const
 {
     if (m_inspectorController)
-        m_inspectorController->stopUserInitiatedProfiling();
-}
-
-void InspectorBackend::enableProfiler(bool always)
-{
-    if (m_inspectorController)
-        m_inspectorController->enableProfiler(always);
-}
-
-void InspectorBackend::disableProfiler(bool always)
-{
-    if (m_inspectorController)
-        m_inspectorController->disableProfiler(always);
-}
-
-bool InspectorBackend::profilerEnabled()
-{
-    if (m_inspectorController)
-        return m_inspectorController->profilerEnabled();
+        return m_inspectorController->debuggerEnabled();
     return false;
 }
 
@@ -337,18 +168,6 @@ void InspectorBackend::disableDebugger(bool always)
         m_inspectorController->disableDebugger(always);
 }
 
-bool InspectorBackend::debuggerEnabled() const
-{
-    if (m_inspectorController)
-        return m_inspectorController->debuggerEnabled();
-    return false;
-}
-
-JavaScriptCallFrame* InspectorBackend::currentCallFrame() const
-{
-    return JavaScriptDebugServer::shared().currentCallFrame();
-}
-
 void InspectorBackend::addBreakpoint(const String& sourceID, unsigned lineNumber, const String& condition)
 {
     intptr_t sourceIDValue = sourceID.toIntPtr();
@@ -367,16 +186,6 @@ void InspectorBackend::removeBreakpoint(const String& sourceID, unsigned lineNum
     JavaScriptDebugServer::shared().removeBreakpoint(sourceIDValue, lineNumber);
 }
 
-bool InspectorBackend::pauseOnExceptions()
-{
-    return JavaScriptDebugServer::shared().pauseOnExceptions();
-}
-
-void InspectorBackend::setPauseOnExceptions(bool pause)
-{
-    JavaScriptDebugServer::shared().setPauseOnExceptions(pause);
-}
-
 void InspectorBackend::pauseInDebugger()
 {
     JavaScriptDebugServer::shared().pauseProgram();
@@ -403,23 +212,95 @@ void InspectorBackend::stepOutOfFunctionInDebugger()
     JavaScriptDebugServer::shared().stepOutOfFunction();
 }
 
+long InspectorBackend::pauseOnExceptionsState()
+{
+    return JavaScriptDebugServer::shared().pauseOnExceptionsState();
+}
+
+void InspectorBackend::setPauseOnExceptionsState(long pauseState)
+{
+    JavaScriptDebugServer::shared().setPauseOnExceptionsState(static_cast<JavaScriptDebugServer::PauseOnExceptionsState>(pauseState));
+}
+
+bool InspectorBackend::profilerEnabled()
+{
+    if (m_inspectorController)
+        return m_inspectorController->profilerEnabled();
+    return false;
+}
+
+void InspectorBackend::enableProfiler(bool always)
+{
+    if (m_inspectorController)
+        m_inspectorController->enableProfiler(always);
+}
+
+void InspectorBackend::disableProfiler(bool always)
+{
+    if (m_inspectorController)
+        m_inspectorController->disableProfiler(always);
+}
+
+void InspectorBackend::startProfiling()
+{
+    if (m_inspectorController)
+        m_inspectorController->startUserInitiatedProfiling();
+}
+
+void InspectorBackend::stopProfiling()
+{
+    if (m_inspectorController)
+        m_inspectorController->stopUserInitiatedProfiling();
+}
+
+void InspectorBackend::getProfileHeaders(long callId)
+{
+    if (m_inspectorController)
+        m_inspectorController->getProfileHeaders(callId);
+}
+
+void InspectorBackend::getProfile(long callId, unsigned uid)
+{
+    if (m_inspectorController)
+        m_inspectorController->getProfile(callId, uid);
+}
+
+JavaScriptCallFrame* InspectorBackend::currentCallFrame() const
+{
+    return JavaScriptDebugServer::shared().currentCallFrame();
+}
 #endif
 
-void InspectorBackend::dispatchOnInjectedScript(long callId, const String& methodName, const String& arguments)
+void InspectorBackend::setInjectedScriptSource(const String& source)
+{
+    if (m_inspectorController)
+        m_inspectorController->injectedScriptHost()->setInjectedScriptSource(source);
+}
+
+void InspectorBackend::dispatchOnInjectedScript(long callId, long injectedScriptId, const String& methodName, const String& arguments, bool async)
 {
     InspectorFrontend* frontend = inspectorFrontend();
     if (!frontend)
         return;
 
-    ScriptFunctionCall function(m_inspectorController->m_scriptState, m_inspectorController->m_injectedScriptObj, "dispatch");
-    function.appendArgument(methodName);
-    function.appendArgument(arguments);
-    bool hadException = false;
-    ScriptValue result = function.call(hadException);
-    if (hadException)
-        frontend->didDispatchOnInjectedScript(callId, "", true);
+    // FIXME: explicitly pass injectedScriptId along with node id to the frontend.
+    bool injectedScriptIdIsNodeId = injectedScriptId <= 0;
+
+    InjectedScript injectedScript;
+    if (injectedScriptIdIsNodeId)
+        injectedScript = m_inspectorController->injectedScriptForNodeId(-injectedScriptId);
     else
-        frontend->didDispatchOnInjectedScript(callId, result.toString(m_inspectorController->m_scriptState), false);
+        injectedScript = m_inspectorController->injectedScriptHost()->injectedScriptForId(injectedScriptId);
+
+    if (injectedScript.hasNoValue())
+        return;
+
+    RefPtr<SerializedScriptValue> result;
+    bool hadException = false;
+    injectedScript.dispatch(callId, methodName, arguments, async, &result, &hadException);
+    if (async)
+        return;  // InjectedScript will return result asynchronously by means of ::reportDidDispatchOnInjectedScript.
+    frontend->didDispatchOnInjectedScript(callId, result.get(), hadException);
 }
 
 void InspectorBackend::getChildNodes(long callId, long nodeId)
@@ -446,64 +327,123 @@ void InspectorBackend::setTextNodeValue(long callId, long nodeId, const String&
         domAgent->setTextNodeValue(callId, nodeId, value);
 }
 
-void InspectorBackend::highlight(long nodeId)
+void InspectorBackend::getEventListenersForNode(long callId, long nodeId)
+{
+    if (InspectorDOMAgent* domAgent = inspectorDOMAgent())
+        domAgent->getEventListenersForNode(callId, nodeId);
+}
+
+void InspectorBackend::copyNode(long nodeId)
+{
+    Node* node = nodeForId(nodeId);
+    if (!node)
+        return;
+    String markup = createMarkup(node);
+    Pasteboard::generalPasteboard()->writePlainText(markup);
+}
+    
+void InspectorBackend::removeNode(long callId, long nodeId)
+{
+    InspectorFrontend* frontend = inspectorFrontend();
+    if (!frontend)
+        return;
+
+    Node* node = nodeForId(nodeId);
+    if (!node) {
+        // Use -1 to denote an error condition.
+        frontend->didRemoveNode(callId, -1);
+        return;
+    }
+
+    Node* parentNode = node->parentNode();
+    if (!parentNode) {
+        frontend->didRemoveNode(callId, -1);
+        return;
+    }
+
+    ExceptionCode code;
+    parentNode->removeChild(node, code);
+    if (code) {
+        frontend->didRemoveNode(callId, -1);
+        return;
+    }
+
+    frontend->didRemoveNode(callId, nodeId);
+}
+
+void InspectorBackend::highlightDOMNode(long nodeId)
 {
     if (Node* node = nodeForId(nodeId))
         m_inspectorController->highlight(node);
 }
 
-Node* InspectorBackend::nodeForId(long nodeId)
-{
-    if (InspectorDOMAgent* domAgent = inspectorDOMAgent())
-        return domAgent->nodeForId(nodeId);
-    return 0;
-}
-
-ScriptValue InspectorBackend::wrapObject(const ScriptValue& object)
+void InspectorBackend::hideDOMNodeHighlight()
 {
     if (m_inspectorController)
-        return m_inspectorController->wrapObject(object);
-    return ScriptValue();
+        m_inspectorController->hideHighlight();
 }
 
-ScriptValue InspectorBackend::unwrapObject(const String& objectId)
+void InspectorBackend::getCookies(long callId)
+{
+    if (!m_inspectorController)
+        return;
+    m_inspectorController->getCookies(callId);
+}
+
+void InspectorBackend::deleteCookie(const String& cookieName, const String& domain)
+{
+    if (!m_inspectorController)
+        return;
+    m_inspectorController->deleteCookie(cookieName, domain);
+}
+
+void InspectorBackend::releaseWrapperObjectGroup(long injectedScriptId, const String& objectGroup)
+{
+    if (!m_inspectorController)
+        return;
+    m_inspectorController->injectedScriptHost()->releaseWrapperObjectGroup(injectedScriptId, objectGroup);
+}
+
+void InspectorBackend::didEvaluateForTestInFrontend(long callId, const String& jsonResult)
 {
     if (m_inspectorController)
-        return m_inspectorController->unwrapObject(objectId);
-    return ScriptValue();
-}
-
-long InspectorBackend::pushNodePathToFrontend(Node* node, bool selectInUI)
-{
-    InspectorFrontend* frontend = inspectorFrontend();
-    InspectorDOMAgent* domAgent = inspectorDOMAgent();
-    if (!domAgent || !frontend)
-        return 0;
-    long id = domAgent->pushNodePathToFrontend(node);
-    if (selectInUI)
-        frontend->updateFocusedNode(id);
-    return id;
-}
-
-void InspectorBackend::addNodesToSearchResult(const String& nodeIds)
-{
-    if (InspectorFrontend* frontend = inspectorFrontend())
-        frontend->addNodesToSearchResult(nodeIds);
+        m_inspectorController->didEvaluateForTestInFrontend(callId, jsonResult);
 }
 
 #if ENABLE(DATABASE)
-void InspectorBackend::selectDatabase(Database* database)
+void InspectorBackend::getDatabaseTableNames(long callId, long databaseId)
 {
-    if (InspectorFrontend* frontend = inspectorFrontend())
-        frontend->selectDatabase(database);
+    if (InspectorFrontend* frontend = inspectorFrontend()) {
+        ScriptArray result = frontend->newScriptArray();
+        Database* database = m_inspectorController->databaseForId(databaseId);
+        if (database) {
+            Vector<String> tableNames = database->tableNames();
+            unsigned length = tableNames.size();
+            for (unsigned i = 0; i < length; ++i)
+                result.set(i, tableNames[i]);
+        }
+        frontend->didGetDatabaseTableNames(callId, result);
+    }
 }
 #endif
 
 #if ENABLE(DOM_STORAGE)
-void InspectorBackend::selectDOMStorage(Storage* storage)
+void InspectorBackend::getDOMStorageEntries(long callId, long storageId)
 {
-    if (InspectorFrontend* frontend = inspectorFrontend())
-        frontend->selectDOMStorage(storage);
+    if (m_inspectorController)
+        m_inspectorController->getDOMStorageEntries(callId, storageId);
+}
+
+void InspectorBackend::setDOMStorageItem(long callId, long storageId, const String& key, const String& value)
+{
+    if (m_inspectorController)
+        m_inspectorController->setDOMStorageItem(callId, storageId, key, value);
+}
+
+void InspectorBackend::removeDOMStorageItem(long callId, long storageId, const String& key)
+{
+    if (m_inspectorController)
+        m_inspectorController->removeDOMStorageItem(callId, storageId, key);
 }
 #endif
 
@@ -521,6 +461,13 @@ InspectorFrontend* InspectorBackend::inspectorFrontend()
     return m_inspectorController->m_frontend.get();
 }
 
+Node* InspectorBackend::nodeForId(long nodeId)
+{
+    if (InspectorDOMAgent* domAgent = inspectorDOMAgent())
+        return domAgent->nodeForId(nodeId);
+    return 0;
+}
+
 } // namespace WebCore
 
 #endif // ENABLE(INSPECTOR)
