@@ -35,7 +35,7 @@
 #include "ScriptState.h"
 #include <wtf/PassOwnPtr.h>
 
-#if ENABLE(JAVASCRIPT_DEBUGGER)
+#if ENABLE(JAVASCRIPT_DEBUGGER) && USE(JSC)
 namespace JSC {
     class JSValue;
     class SourceCode;
@@ -50,30 +50,32 @@ namespace WebCore {
     class InspectorController;
     class InspectorResource;
     class Node;
-    class ScriptFunctionCall;
     class ScriptString;
+    class SerializedScriptValue;
     class Storage;
 
-    class InspectorFrontend {
+    class InspectorFrontend : public Noncopyable {
     public:
-        InspectorFrontend(InspectorController* inspectorController, ScriptState*, ScriptObject webInspector);
+        InspectorFrontend(InspectorController* inspectorController, ScriptObject webInspector);
         ~InspectorFrontend();
 
         ScriptArray newScriptArray();
         ScriptObject newScriptObject();
 
         void didCommitLoad();
-        void addConsoleMessage(const ScriptObject& messageObj, const Vector<ScriptString>& frames, const Vector<ScriptValue> wrappedArguments, const String& message);
-        void updateConsoleMessageRepeatCount(const int count);
+
+        void populateFrontendSettings(const String& settings);
+
+        void updateConsoleMessageExpiredCount(unsigned count);
+        void addConsoleMessage(const ScriptObject& messageObj, const Vector<ScriptString>& frames, ScriptState*, const Vector<ScriptValue> arguments, const String& message);
+        void updateConsoleMessageRepeatCount(unsigned count);
         void clearConsoleMessages();
 
-        bool addResource(long long identifier, const ScriptObject& resourceObj);
-        bool updateResource(long long identifier, const ScriptObject& resourceObj);
-        void removeResource(long long identifier);
+        bool updateResource(unsigned long identifier, const ScriptObject& resourceObj);
+        void removeResource(unsigned long identifier);
+        void didGetResourceContent(int callId, const String& content);
 
-        void addCookieDomain(String);
-
-        void updateFocusedNode(long long nodeId);
+        void updateFocusedNode(long nodeId);
         void setAttachedWindow(bool attached);
         void showPanel(int panel);
         void populateInterface();
@@ -82,7 +84,7 @@ namespace WebCore {
         void resourceTrackingWasEnabled();
         void resourceTrackingWasDisabled();
 
-#if ENABLE(JAVASCRIPT_DEBUGGER)
+#if ENABLE(JAVASCRIPT_DEBUGGER) && USE(JSC)
         void attachDebuggerWhenShown();
         void debuggerWasEnabled();
         void debuggerWasDisabled();
@@ -94,7 +96,7 @@ namespace WebCore {
         void setRecordingProfile(bool isProfiling);
         void didGetProfileHeaders(int callId, const ScriptArray& headers);
         void didGetProfile(int callId, const ScriptValue& profile);
-        void pausedScript(const ScriptValue& callFrames);
+        void pausedScript(SerializedScriptValue* callFrames);
         void resumedScript();
 #endif
 
@@ -127,21 +129,22 @@ namespace WebCore {
 
         void timelineProfilerWasStarted();
         void timelineProfilerWasStopped();
-        void addItemToTimeline(const ScriptObject& itemObj);
+        void addRecordToTimeline(const ScriptObject&);
 
         void didGetCookies(int callId, const ScriptArray& cookies, const String& cookiesString);
-        void didDispatchOnInjectedScript(int callId, const String& result, bool isException);
+        void didDispatchOnInjectedScript(int callId, SerializedScriptValue* result, bool isException);
 
         void addNodesToSearchResult(const String& nodeIds);
 
-        ScriptState* scriptState() const { return m_scriptState; }
+        void contextMenuItemSelected(int itemId);
+        void contextMenuCleared();
+
+        ScriptState* scriptState() const { return m_webInspector.scriptState(); }
 
         void evaluateForTestInFrontend(int callId, const String& script);
     private:
-        PassOwnPtr<ScriptFunctionCall> newFunctionCall(const String& functionName);
         void callSimpleFunction(const String& functionName);
         InspectorController* m_inspectorController;
-        ScriptState* m_scriptState;
         ScriptObject m_webInspector;
     };
 
