@@ -1,6 +1,7 @@
 /*
  * Copyright (C) 2006, 2007, 2008 Apple Inc. All rights reserved.
  * Copyright (C) 2007 Alp Toker <alp@atoker.com>
+ * Copyright (C) 2008 Torch Mobile, Inc.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -35,8 +36,15 @@
 #include <wtf/Vector.h>
 
 #if PLATFORM(CG)
+
+#ifdef BUILDING_ON_TIGER
 typedef struct CGShading* CGShadingRef;
 typedef CGShadingRef PlatformGradient;
+#else
+typedef struct CGGradient* CGGradientRef;
+typedef CGGradientRef PlatformGradient;
+#endif
+
 #elif PLATFORM(QT)
 QT_BEGIN_NAMESPACE
 class QGradient;
@@ -73,8 +81,17 @@ namespace WebCore {
 
         void getColor(float value, float* r, float* g, float* b, float* a) const;
 
+#if OS(WINCE) && !PLATFORM(QT)
+        const FloatPoint& p0() const { return m_p0; }
+        const FloatPoint& p1() const { return m_p1; }
+        float r0() const { return m_r0; }
+        float r1() const { return m_r1; }
+        bool isRadial() const { return m_radial; }
+        struct ColorStop;
+        const Vector<ColorStop>& getStops() const;
+#else
         PlatformGradient platformGradient();
-
+#endif
         struct ColorStop {
             float stop;
             float red;
@@ -87,7 +104,7 @@ namespace WebCore {
         };
 
         void setStopsSorted(bool s) { m_stopsSorted = s; }
-
+        
         void setSpreadMethod(GradientSpreadMethod);
         GradientSpreadMethod spreadMethod() { return m_spreadMethod; }
         void setGradientSpaceTransform(const TransformationMatrix& gradientSpaceTransformation);
@@ -95,8 +112,13 @@ namespace WebCore {
         TransformationMatrix gradientSpaceTransform() { return m_gradientSpaceTransformation; }
 
         virtual void fill(GraphicsContext*, const FloatRect&);
+        virtual void adjustParametersForTiledDrawing(IntSize& size, FloatRect& srcRect);
+
         void setPlatformGradientSpaceTransform(const TransformationMatrix& gradientSpaceTransformation);
 
+#if PLATFORM(CG)
+        void paint(GraphicsContext*);
+#endif
     private:
         Gradient(const FloatPoint& p0, const FloatPoint& p1);
         Gradient(const FloatPoint& p0, float r0, const FloatPoint& p1, float r1);
@@ -105,6 +127,7 @@ namespace WebCore {
         void platformDestroy();
 
         int findStop(float value) const;
+        void sortStopsIfNecessary();
 
         bool m_radial;
         FloatPoint m_p0;
