@@ -1,8 +1,6 @@
 /*
     Copyright (C) 2007 Nikolas Zimmermann <zimmermann@kde.org>
 
-    This file is part of the KDE project
-
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
     License as published by the Free Software Foundation; either
@@ -38,9 +36,9 @@ namespace WebCore {
 SVGTextPathElement::SVGTextPathElement(const QualifiedName& tagName, Document* doc)
     : SVGTextContentElement(tagName, doc)
     , SVGURIReference()
-    , m_startOffset(this, SVGNames::startOffsetAttr, LengthModeOther)
-    , m_method(this, SVGNames::methodAttr, SVG_TEXTPATH_METHODTYPE_ALIGN)
-    , m_spacing(this, SVGNames::spacingAttr, SVG_TEXTPATH_SPACINGTYPE_EXACT)
+    , m_startOffset(LengthModeOther)
+    , m_method(SVG_TEXTPATH_METHODTYPE_ALIGN)
+    , m_spacing(SVG_TEXTPATH_SPACINGTYPE_EXACT)
 {
 }
 
@@ -71,6 +69,28 @@ void SVGTextPathElement::parseMappedAttribute(MappedAttribute* attr)
     }
 }
 
+void SVGTextPathElement::synchronizeProperty(const QualifiedName& attrName)
+{
+    SVGTextContentElement::synchronizeProperty(attrName);
+
+    if (attrName == anyQName()) {
+        synchronizeStartOffset();
+        synchronizeMethod();
+        synchronizeSpacing();
+        synchronizeHref();
+        return;
+    }
+
+    if (attrName == SVGNames::startOffsetAttr)
+        synchronizeStartOffset();
+    else if (attrName == SVGNames::methodAttr)
+        synchronizeMethod();
+    else if (attrName == SVGNames::spacingAttr)
+        synchronizeSpacing();
+    else if (SVGURIReference::isKnownAttribute(attrName))
+        synchronizeHref();
+}
+
 RenderObject* SVGTextPathElement::createRenderer(RenderArena* arena, RenderStyle*)
 {
     return new (arena) RenderSVGTextPath(this);
@@ -103,5 +123,3 @@ void SVGTextPathElement::insertedIntoDocument()
 }
 
 #endif // ENABLE(SVG)
-
-// vim:ts=4:noet
