@@ -3,6 +3,7 @@
  * Copyright (C) 2007 Eric Seidel <eric@webkit.org>
  * Copyright (C) 2008 Nuanti Ltd.
  * Copyright (C) 2009 Jan Michael Alonzo <jmalonzo@gmail.com>
+ * Copyright (C) 2009 Collabora Ltd.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -48,10 +49,14 @@
 extern "C" {
 bool webkit_web_frame_pause_animation(WebKitWebFrame* frame, const gchar* name, double time, const gchar* element);
 bool webkit_web_frame_pause_transition(WebKitWebFrame* frame, const gchar* name, double time, const gchar* element);
+bool webkit_web_frame_pause_svg_animation(WebKitWebFrame* frame, const gchar* name, double time, const gchar* element);
 unsigned int webkit_web_frame_number_of_active_animations(WebKitWebFrame* frame);
 void webkit_application_cache_set_maximum_size(unsigned long long size);
 unsigned int webkit_worker_thread_count(void);
 void webkit_white_list_access_from_origin(const gchar* sourceOrigin, const gchar* destinationProtocol, const gchar* destinationHost, bool allowDestinationSubdomains);
+gchar* webkit_web_frame_counter_value_for_element_by_id(WebKitWebFrame* frame, const gchar* id);
+int webkit_web_frame_page_number_for_element_by_id(WebKitWebFrame* frame, const gchar* id, float pageWidth, float pageHeight);
+void webkit_web_inspector_execute_script(WebKitWebInspector* inspector, long callId, const gchar* script);
 }
 
 static gchar* copyWebSettingKey(gchar* preferenceKey)
@@ -64,6 +69,8 @@ static gchar* copyWebSettingKey(gchar* preferenceKey)
         keyTable = g_hash_table_new(g_str_hash, g_str_equal);
         g_hash_table_insert(keyTable, g_strdup("WebKitJavaScriptEnabled"), g_strdup("enable-scripts"));
         g_hash_table_insert(keyTable, g_strdup("WebKitDefaultFontSize"), g_strdup("default-font-size"));
+        g_hash_table_insert(keyTable, g_strdup("WebKitEnableCaretBrowsing"), g_strdup("enable-caret-browsing"));
+        g_hash_table_insert(keyTable, g_strdup("WebKitUsesPageCachePreferenceKey"), g_strdup("enable-page-cache"));
     }
 
     return g_strdup(static_cast<gchar*>(g_hash_table_lookup(keyTable, preferenceKey)));
@@ -118,11 +125,30 @@ void LayoutTestController::display()
     displayWebView();
 }
 
+JSRetainPtr<JSStringRef> LayoutTestController::counterValueForElementById(JSStringRef id)
+{
+    gchar* idGChar = JSStringCopyUTF8CString(id);
+    gchar* counterValueGChar = webkit_web_frame_counter_value_for_element_by_id(mainFrame, idGChar);
+    g_free(idGChar);
+    if (!counterValueGChar)
+        return 0;
+    JSRetainPtr<JSStringRef> counterValue(Adopt, JSStringCreateWithUTF8CString(counterValueGChar));
+    return counterValue;
+}
+
 void LayoutTestController::keepWebHistory()
 {
     // FIXME: implement
 }
 
+int LayoutTestController::pageNumberForElementById(JSStringRef id, float pageWidth, float pageHeight)
+{
+    gchar* idGChar = JSStringCopyUTF8CString(id);
+    int pageNumber = webkit_web_frame_page_number_for_element_by_id(mainFrame, idGChar, pageWidth, pageHeight);
+    g_free(idGChar);
+    return pageNumber;
+}
+
 size_t LayoutTestController::webHistoryItemCount()
 {
     // FIXME: implement
@@ -210,7 +236,18 @@ void LayoutTestController::setMainFrameIsFirstResponder(bool flag)
 
 void LayoutTestController::setTabKeyCyclesThroughElements(bool cycles)
 {
-    // FIXME: implement
+    WebKitWebView* webView = webkit_web_frame_get_web_view(mainFrame);
+    WebKitWebSettings* settings = webkit_web_view_get_settings(webView);
+    g_object_set(G_OBJECT(settings), "tab-key-cycles-through-elements", cycles, NULL);
+}
+
+void LayoutTestController::setTimelineProfilingEnabled(bool flag)
+{
+    WebKitWebView* view = webkit_web_frame_get_web_view(mainFrame);
+    ASSERT(view);
+
+    WebKitWebInspector* inspector = webkit_web_view_get_inspector(view);
+    g_object_set(G_OBJECT(inspector), "timeline-profiling-enabled", flag, NULL);
 }
 
 void LayoutTestController::setUseDashboardCompatibilityMode(bool flag)
@@ -291,6 +328,15 @@ void LayoutTestController::setXSSAuditorEnabled(bool flag)
     g_object_set(G_OBJECT(settings), "enable-xss-auditor", flag, NULL);
 }
 
+void LayoutTestController::setAllowUniversalAccessFromFileURLs(bool flag)
+{
+    WebKitWebView* view = webkit_web_frame_get_web_view(mainFrame);
+    ASSERT(view);
+
+    WebKitWebSettings* settings = webkit_web_view_get_settings(view);
+    g_object_set(G_OBJECT(settings), "enable-universal-access-from-file-uris", flag, NULL);
+}
+
 void LayoutTestController::setAuthorAndUserStylesEnabled(bool flag)
 {
     // FIXME: implement
@@ -389,6 +435,11 @@ void LayoutTestController::setDatabaseQuota(unsigned long long quota)
     webkit_security_origin_set_web_database_quota(origin, quota);
 }
 
+void LayoutTestController::setDomainRelaxationForbiddenForURLScheme(bool, JSStringRef)
+{
+    // FIXME: implement
+}
+
 void LayoutTestController::setAppCacheMaximumSize(unsigned long long size)
 {
     webkit_application_cache_set_maximum_size(size);
@@ -414,6 +465,16 @@ bool LayoutTestController::pauseTransitionAtTimeOnElementWithId(JSStringRef prop
     return returnValue;
 }
 
+bool LayoutTestController::sampleSVGAnimationForElementAtTime(JSStringRef animationId, double time, JSStringRef elementId)
+{    
+    gchar* name = JSStringCopyUTF8CString(animationId);
+    gchar* element = JSStringCopyUTF8CString(elementId);
+    bool returnValue = webkit_web_frame_pause_svg_animation(mainFrame, name, time, element);
+    g_free(name);
+    g_free(element);
+    return returnValue;
+}
+
 unsigned LayoutTestController::numberOfActiveAnimations() const
 {
     return webkit_web_frame_number_of_active_animations(mainFrame);
@@ -444,9 +505,12 @@ void LayoutTestController::overridePreference(JSStringRef key, JSStringRef value
             g_value_transform(const_cast<GValue*>(&stringValue), &propValue);
             g_object_set_property(G_OBJECT(settings), webSettingKey, const_cast<GValue*>(&propValue));
         } else if (G_VALUE_HOLDS_BOOLEAN(&propValue)) {
+            char* lowered = g_utf8_strdown(strValue, -1);
             g_object_set(G_OBJECT(settings), webSettingKey,
-                         g_str_equal(g_utf8_strdown(strValue, -1), "true"),
+                         g_str_equal(lowered, "true")
+                         || g_str_equal(strValue, "1"),
                          NULL);
+            g_free(lowered);
         } else if (G_VALUE_HOLDS_INT(&propValue)) {
             std::string str(strValue);
             std::stringstream ss(str);
@@ -474,17 +538,32 @@ void LayoutTestController::addUserStyleSheet(JSStringRef source)
 
 void LayoutTestController::showWebInspector()
 {
-    // FIXME: Implement this.
+    WebKitWebView* webView = webkit_web_frame_get_web_view(mainFrame);
+    WebKitWebSettings* webSettings = webkit_web_view_get_settings(webView);
+    WebKitWebInspector* inspector = webkit_web_view_get_inspector(webView);
+
+    g_object_set(webSettings, "enable-developer-extras", TRUE, NULL);
+    webkit_web_inspector_show(inspector);
 }
 
 void LayoutTestController::closeWebInspector()
 {
-    // FIXME: Implement this.
+    WebKitWebView* webView = webkit_web_frame_get_web_view(mainFrame);
+    WebKitWebSettings* webSettings = webkit_web_view_get_settings(webView);
+    WebKitWebInspector* inspector = webkit_web_view_get_inspector(webView);
+
+    webkit_web_inspector_close(inspector);
+    g_object_set(webSettings, "enable-developer-extras", FALSE, NULL);
 }
 
 void LayoutTestController::evaluateInWebInspector(long callId, JSStringRef script)
 {
-    // FIXME: Implement this.
+    WebKitWebView* webView = webkit_web_frame_get_web_view(mainFrame);
+    WebKitWebInspector* inspector = webkit_web_view_get_inspector(webView);
+    char* scriptString = JSStringCopyUTF8CString(script);
+
+    webkit_web_inspector_execute_script(inspector, callId, scriptString);
+    g_free(scriptString);
 }
 
 void LayoutTestController::evaluateScriptInIsolatedWorld(unsigned worldID, JSObjectRef globalObject, JSStringRef script)
